"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var entities_1 = require("dxf-doc/entities");
var hatch_1 = require("dxf-doc/entities/hatch");
var __1 = require("..");
// http://gohtx.com/acadcolors.php
var COLORS = [
    ['black', '#000', '#000000'],
    ['red', '#f00', '#ff0000'],
    ['yellow', '#ff0', '#ffff00'],
    ['lime', '#0f0', '#00ff00'],
    ['cyan', '#0ff', '#00ffff'],
    ['blue', '#00f', '#0000ff'],
    ['magenta', '#f0f', '#ff00ff'],
    ['white', '#fff', '#ffffff'],
    ['gray', '#888', '#888888'],
    ['darkgray', '#c0c0c0']
];
var AbstractDxfBuilder = /** @class */ (function () {
    function AbstractDxfBuilder(document) {
        this.document = document;
        this.ltypes = new Set();
        this.patterns = new Map();
        this.textStyles = new Map();
    }
    AbstractDxfBuilder.prototype.getStyle = function (attributes) {
        var style = {};
        if (attributes) {
            style.stroke = attributes.stroke;
            style.strokeWidth = attributes['stroke-width'] ? +attributes['stroke-width'] : attributes['stroke-width'];
            style.strokeDashArray = attributes['stroke-dasharray'] ?
                Array.isArray(attributes['stroke-dasharray']) ? attributes['stroke-dasharray'].map(function (x) { return +x; }) :
                    typeof attributes['stroke-dasharray'] === 'string' ? (attributes['stroke-dasharray'] + '').split(' ').map(function (x) { return +x; }) : undefined
                : undefined;
            style.fill = attributes.fill;
            if (attributes.font) {
                var match = (attributes.font + '').match(/(?:(normal|italic)\s)?(?:(normal|bold)\s)?(\d+)\s(.+)/);
                if (match) {
                    style.fontItalic = match[1] === 'italic';
                    style.fontBold = match[2] === 'bold';
                    style.fontSize = +match[3];
                    style.fontFamily = match[4];
                }
            }
            style.fontItalic = attributes['font-style'] === 'italic';
            style.fontBold = attributes['font-weight'] === 'bold';
            if (attributes['font-size']) {
                style.fontSize = +attributes['font-size'];
            }
            if (attributes['font-family']) {
                style.fontFamily = attributes['font-family'];
            }
        }
        return style;
    };
    AbstractDxfBuilder.prototype.getLType = function (style) {
        var strokeDashArray = style.strokeDashArray;
        var result = strokeDashArray ? 'DASH_' + strokeDashArray.map(function (x) { return x.toFixed(2); }).join('_') : undefined;
        if (result && !this.ltypes.has(result)) {
            this.document.addLineType(result, strokeDashArray);
            this.ltypes.add(result);
        }
        return result;
    };
    AbstractDxfBuilder.prototype.getLineWeight = function (style) {
        var strokeWidth = style.strokeWidth;
        return strokeWidth ? Math.round(strokeWidth * 100) : undefined;
    };
    AbstractDxfBuilder.prototype.getColor = function (style, key) {
        var color = style[key];
        if (!color) {
            return undefined;
        }
        var colorIndex = COLORS.findIndex(function (values) { return values.some(function (val) { return val === color; }); });
        return colorIndex >= 0 ? colorIndex : undefined;
    };
    AbstractDxfBuilder.prototype.getFillPattern = function (style) {
        var pattern;
        if (style.fill) {
            var match = style.fill.match(/^url\(#(.+)\)$/);
            if (match) {
                pattern = match[1];
            }
        }
        return pattern;
    };
    AbstractDxfBuilder.prototype.getTextStyle = function (style) {
        var fontFamily = style.fontFamily, fontItalic = style.fontItalic, fontBold = style.fontBold;
        var baseName = fontFamily ? fontFamily : 'Standard';
        var flags = "" + (fontBold ? 'b' : '') + (fontItalic ? 'i' : '');
        var result = flags ? baseName + "_" + flags : baseName;
        if (result !== 'Standard' && !this.textStyles.has(result)) {
            this.document.addStyle(result, fontFamily);
        }
        return result;
    };
    AbstractDxfBuilder.prototype.line = function (x1, y1, x2, y2, attributes) {
        var style = this.getStyle(attributes);
        if (style.stroke !== 'none') {
            var entity = new entities_1.Line(this.document, x1, this.convertY(y1), x2, this.convertY(y2));
            entity.ltype = this.getLType(style);
            entity.lineWeight = this.getLineWeight(style);
            entity.color = this.getColor(style, 'stroke');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.rect = function (x, y, width, height, attributes) {
        this.polygon([{ x: x, y: y }, { x: x + width, y: y }, { x: x + width, y: y + height }, { x: x, y: y + height }]);
    };
    AbstractDxfBuilder.prototype.circle = function (cx, cy, r, attributes) {
        var style = this.getStyle(attributes);
        if (style.fill && style.fill !== 'none') {
            var patternName = this.getFillPattern(style);
            var boundary = new hatch_1.EdgeBoundaryPath();
            boundary.acr(cx, this.convertY(cy), r, 0, 360, true);
            var pattern = patternName ? this.patterns.get(patternName) : undefined;
            var entity = new entities_1.Hatch(this.document, [boundary], pattern);
            entity.color = this.getColor(style, 'fill');
            this.document.addEntity(entity);
        }
        if (style.stroke !== 'none') {
            var entity = new entities_1.Circle(this.document, cx, this.convertY(cy), r);
            entity.ltype = this.getLType(style);
            entity.lineWeight = this.getLineWeight(style);
            entity.color = this.getColor(style, 'stroke');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.ellipse = function (cx, cy, rx, ry, attributes) {
        var style = this.getStyle(attributes);
        if (style.fill && style.fill !== 'none') {
            var patternName = this.getFillPattern(style);
            var boundary = new hatch_1.EdgeBoundaryPath();
            if (rx > ry) {
                boundary.ellipse(cx, this.convertY(cy), rx, 0, ry / rx * 100, 0, 360, true);
            }
            else {
                boundary.ellipse(cx, this.convertY(cy), 0, ry, rx / ry * 100, 0, 360, true);
            }
            var pattern = patternName ? this.patterns.get(patternName) : undefined;
            var entity = new entities_1.Hatch(this.document, [boundary], pattern);
            entity.color = this.getColor(style, 'fill');
            this.document.addEntity(entity);
        }
        if (style.stroke !== 'none') {
            var entity = void 0;
            if (rx > ry) {
                entity = new entities_1.Ellipse(this.document, cx, this.convertY(cy), rx, 0, ry / rx);
            }
            else {
                entity = new entities_1.Ellipse(this.document, cx, this.convertY(cy), 0, ry, rx / ry);
            }
            entity.ltype = this.getLType(style);
            entity.lineWeight = this.getLineWeight(style);
            entity.color = this.getColor(style, 'stroke');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.polyline = function (pts, attributes) {
        var _this = this;
        var style = this.getStyle(attributes);
        if (style.stroke !== 'none') {
            var entity = new entities_1.LwPolyline(this.document, pts.map(function (p) { return [p.x, _this.convertY(p.y)]; }), false);
            entity.ltype = this.getLType(style);
            entity.lineWeight = this.getLineWeight(style);
            entity.color = this.getColor(style, 'stroke');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.polygon = function (pts, attributes) {
        var _this = this;
        var style = this.getStyle(attributes);
        var dxfPts = pts.map(function (p) { return [p.x, _this.convertY(p.y)]; });
        if (style.fill && style.fill !== 'none') {
            var patternName = this.getFillPattern(style);
            var boundary = new hatch_1.PolylineBoundaryPath(dxfPts);
            var pattern = patternName ? this.patterns.get(patternName) : undefined;
            var entity = new entities_1.Hatch(this.document, [boundary], pattern);
            entity.color = this.getColor(style, 'fill');
            this.document.addEntity(entity);
        }
        if (style.stroke !== 'none') {
            var entity = new entities_1.LwPolyline(this.document, dxfPts, true);
            entity.ltype = this.getLType(style);
            entity.lineWeight = this.getLineWeight(style);
            entity.color = this.getColor(style, 'stroke');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.polybezier = function (pts, attributes) {
        pts = __1.Geometry.approximatePolybezier(pts, __1.DRAWING_SETTINGS.APPROX_CURVE_MAX_DEVIATION);
        this.polyline(pts);
    };
    AbstractDxfBuilder.prototype.curve = function (pts, closed, smoothing, attributes) {
        pts = __1.Geometry.curveToPolybezier(pts, closed || false, smoothing);
        this.polybezier(pts);
    };
    AbstractDxfBuilder.prototype.text = function (text, x, y, align, attributes) {
        var style = this.getStyle(attributes);
        if (style.fill !== 'none') {
            var defaultFontSize = 16;
            var fontSize = style.fontSize || defaultFontSize;
            var entity = new entities_1.Text(this.document, text, fontSize, [x, this.convertY(y)], align);
            entity.style = this.getTextStyle(style);
            entity.color = this.getColor(style, 'fill');
            this.document.addEntity(entity);
        }
    };
    AbstractDxfBuilder.prototype.region = function (region, attributes) {
        throw new Error("Method not implemented.");
    };
    AbstractDxfBuilder.prototype.path = function (path, attributes) {
        throw new Error("Method not implemented.");
    };
    AbstractDxfBuilder.prototype.createRegion = function () {
        throw new Error("Method not implemented.");
    };
    AbstractDxfBuilder.prototype.createPath = function () {
        throw new Error("Method not implemented.");
    };
    AbstractDxfBuilder.prototype.addPattern = function (pattern) {
        this.patterns.set(pattern.name, pattern);
    };
    return AbstractDxfBuilder;
}());
exports.AbstractDxfBuilder = AbstractDxfBuilder;
